@@ -2,7 +2,7 @@
 #
 # p7zr library
 #
-# Copyright (c) 2019 Hiroshi Miura <miurahr@linux.com>
+# Copyright (c) 2019-2021 Hiroshi Miura <miurahr@linux.com>
 # Copyright (c) 2004-2015 by Joachim Bauch, mail@joachim-bauch.de
 #
 # This library is free software; you can redistribute it and/or
@@ -20,8 +20,6 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 #
-
-import _hashlib  # type: ignore  # noqa
 import ctypes
 import os
 import pathlib
@@ -32,37 +30,38 @@ import zlib
 from datetime import datetime, timedelta, timezone, tzinfo
 from typing import BinaryIO, Optional, Union
 
+import _hashlib  # type: ignore  # noqa
+
 import py7zr.win32compat
 
 
-def calculate_crc32(data: bytes, value: Optional[int] = None, blocksize: int = 1024 * 1024) -> int:
+def calculate_crc32(data: bytes, value: int = 0, blocksize: int = 1024 * 1024) -> int:
     """Calculate CRC32 of strings with arbitrary lengths."""
-    length = len(data)
-    pos = blocksize
-    if value:
-        value = zlib.crc32(data[:pos], value)
+    if len(data) <= blocksize:
+        value = zlib.crc32(data, value)
     else:
-        value = zlib.crc32(data[:pos])
-    while pos < length:
-        value = zlib.crc32(data[pos:pos + blocksize], value)
-        pos += blocksize
-
-    return value & 0xffffffff
+        length = len(data)
+        pos = blocksize
+        value = zlib.crc32(data[:pos], value)
+        while pos < length:
+            value = zlib.crc32(data[pos : pos + blocksize], value)
+            pos += blocksize
+    return value & 0xFFFFFFFF
 
 
 def _calculate_key1(password: bytes, cycles: int, salt: bytes, digest: str) -> bytes:
-    """Calculate 7zip AES encryption key. Base implementation. """
-    if digest not in ('sha256'):
-        raise ValueError('Unknown digest method for password protection.')
-    assert cycles <= 0x3f
-    if cycles == 0x3f:
+    """Calculate 7zip AES encryption key. Base implementation."""
+    if digest not in ("sha256"):
+        raise ValueError("Unknown digest method for password protection.")
+    assert cycles <= 0x3F
+    if cycles == 0x3F:
         ba = bytearray(salt + password + bytes(32))
-        key = bytes(ba[:32])  # type: bytes
+        key: bytes = bytes(ba[:32])
     else:
         rounds = 1 << cycles
         m = _hashlib.new(digest)
         for round in range(rounds):
-            m.update(salt + password + round.to_bytes(8, byteorder='little', signed=False))
+            m.update(salt + password + round.to_bytes(8, byteorder="little", signed=False))
         key = m.digest()[:32]
     return key
 
@@ -70,11 +69,11 @@ def _calculate_key1(password: bytes, cycles: int, salt: bytes, digest: str) -> b
 def _calculate_key2(password: bytes, cycles: int, salt: bytes, digest: str):
     """Calculate 7zip AES encryption key.
     It utilize ctypes and memoryview buffer and zero-copy technology on Python."""
-    if digest not in ('sha256'):
-        raise ValueError('Unknown digest method for password protection.')
-    assert cycles <= 0x3f
-    if cycles == 0x3f:
-        key = bytes(bytearray(salt + password + bytes(32))[:32])  # type: bytes
+    if digest not in ("sha256"):
+        raise ValueError("Unknown digest method for password protection.")
+    assert cycles <= 0x3F
+    if cycles == 0x3F:
+        key: bytes = bytes(bytearray(salt + password + bytes(32))[:32])
     else:
         rounds = 1 << cycles
         m = _hashlib.new(digest)
@@ -83,8 +82,8 @@ def _calculate_key2(password: bytes, cycles: int, salt: bytes, digest: str):
         class RoundBuf(ctypes.LittleEndianStructure):
             _pack_ = 1
             _fields_ = [
-                ('saltpassword', ctypes.c_ubyte * length),
-                ('round', ctypes.c_uint64)
+                ("saltpassword", ctypes.c_ubyte * length),
+                ("round", ctypes.c_uint64),
             ]
 
         buf = RoundBuf()
@@ -102,12 +101,12 @@ def _calculate_key2(password: bytes, cycles: int, salt: bytes, digest: str):
 def _calculate_key3(password: bytes, cycles: int, salt: bytes, digest: str) -> bytes:
     """Calculate 7zip AES encryption key.
     Concat values in order to reduce number of calls of Hash.update()."""
-    if digest not in ('sha256'):
-        raise ValueError('Unknown digest method for password protection.')
-    assert cycles <= 0x3f
-    if cycles == 0x3f:
+    if digest not in ("sha256"):
+        raise ValueError("Unknown digest method for password protection.")
+    assert cycles <= 0x3F
+    if cycles == 0x3F:
         ba = bytearray(salt + password + bytes(32))
-        key = bytes(ba[:32])  # type: bytes
+        key: bytes = bytes(ba[:32])
     else:
         cat_cycle = 6
         if cycles > cat_cycle:
@@ -121,13 +120,19 @@ def _calculate_key3(password: bytes, cycles: int, salt: bytes, digest: str) -> b
         s = 0  # type: int  # (0..stages) * rounds
         if platform.python_implementation() == "PyPy":
             for _ in range(stages):
-                m.update(memoryview(b''.join([saltpassword + (s + i).to_bytes(8, byteorder='little', signed=False)
-                                              for i in range(rounds)])))
+                m.update(
+                    memoryview(
+                        b"".join(
+                            [saltpassword + (s + i).to_bytes(8, byteorder="little", signed=False) for i in range(rounds)]
+                        )
+                    )
+                )
                 s += rounds
         else:
             for _ in range(stages):
-                m.update(b''.join([saltpassword + (s + i).to_bytes(8, byteorder='little', signed=False)
-                                   for i in range(rounds)]))
+                m.update(
+                    b"".join([saltpassword + (s + i).to_bytes(8, byteorder="little", signed=False) for i in range(rounds)])
+                )
                 s += rounds
         key = m.digest()[:32]
 
@@ -166,14 +171,13 @@ DSTDIFF = DSTOFFSET - STDOFFSET
 
 
 class LocalTimezone(tzinfo):
-
     def fromutc(self, dt):
         assert dt.tzinfo is self
         stamp = (dt - datetime(1970, 1, 1, tzinfo=self)) // SECOND
         args = _time.localtime(stamp)[:6]
-        dst_diff = DSTDIFF // SECOND
+        # dst_diff = DSTDIFF // SECOND
         # Detect fold
-        fold = (args == _time.localtime(stamp - dst_diff))
+        # fold = args == _time.localtime(stamp - dst_diff)
         return datetime(*args, microsecond=dt.microsecond, tzinfo=self)
 
     def utcoffset(self, dt):
@@ -192,9 +196,17 @@ class LocalTimezone(tzinfo):
         return _time.tzname[self._isdst(dt)]
 
     def _isdst(self, dt):
-        tt = (dt.year, dt.month, dt.day,
-              dt.hour, dt.minute, dt.second,
-              dt.weekday(), 0, 0)
+        tt = (
+            dt.year,
+            dt.month,
+            dt.day,
+            dt.hour,
+            dt.minute,
+            dt.second,
+            dt.weekday(),
+            0,
+            0,
+        )
         stamp = _time.mktime(tt)
         tt = _time.localtime(stamp)
         return tt.tm_isdst > 0
@@ -224,7 +236,10 @@ class ArchiveTimestamp(int):
     """Windows FILETIME timestamp."""
 
     def __repr__(self):
-        return '%s(%d)' % (type(self).__name__, self)
+        return "%s(%d)" % (type(self).__name__, self)
+
+    def __index__(self):
+        return self.__int__()
 
     def totimestamp(self) -> float:
         """Convert 7z FILETIME to Python timestamp."""
@@ -239,13 +254,17 @@ class ArchiveTimestamp(int):
     def from_datetime(val):
         return ArchiveTimestamp((val - TIMESTAMP_ADJUST) * 10000000.0)
 
+    @staticmethod
+    def from_now():
+        return ArchiveTimestamp((_time.time() - TIMESTAMP_ADJUST) * 10000000.0)
+
 
 def islink(path):
     """
     Cross-platform islink implementation.
     Supports Windows NT symbolic links and reparse points.
     """
-    is_symlink = os.path.islink(path)
+    is_symlink = os.path.islink(str(path))
     if sys.version_info >= (3, 8) or sys.platform != "win32" or sys.getwindowsversion()[0] < 6:
         return is_symlink
     # special check for directory junctions which py38 does.
@@ -263,9 +282,8 @@ def readlink(path: Union[str, pathlib.Path], *, dir_fd=None) -> Union[str, pathl
     When called with Path object, return also Path object.
     When called with path argument as bytes, return result as a bytes.
     """
-    is_path_pathlib = isinstance(path, pathlib.Path)
     if sys.version_info >= (3, 9):
-        if is_path_pathlib and dir_fd is None:
+        if isinstance(path, pathlib.Path) and dir_fd is None:
             return path.readlink()
         else:
             return os.readlink(path, dir_fd=dir_fd)
@@ -274,17 +292,18 @@ def readlink(path: Union[str, pathlib.Path], *, dir_fd=None) -> Union[str, pathl
         # Hack to handle a wrong type of results
         if isinstance(res, bytes):
             res = os.fsdecode(res)
-        if is_path_pathlib:
+        if isinstance(path, pathlib.Path):
             return pathlib.Path(res)
         else:
             return res
     elif not os.path.exists(str(path)):
-        raise OSError(22, 'Invalid argument', path)
+        raise OSError(22, "Invalid argument", path)
     return py7zr.win32compat.readlink(path)
 
 
 class MemIO:
     """pathlib.Path-like IO class to write memory(io.Bytes)"""
+
     def __init__(self, buf: BinaryIO):
         self._buf = buf
 
@@ -336,7 +355,7 @@ class NullIO:
         if length is not None:
             return bytes(length)
         else:
-            return b''
+            return b""
 
     def close(self):
         pass
@@ -366,20 +385,16 @@ class BufferOverflow(Exception):
 
 
 class Buffer:
-
     def __init__(self, size: int = 16):
-        self._size = size
         self._buf = bytearray(size)
         self._buflen = 0
         self.view = memoryview(self._buf[0:0])
 
     def add(self, data: Union[bytes, bytearray, memoryview]):
         length = len(data)
-        if length + self._buflen > self._size:
-            raise BufferOverflow()
-        self._buf[self._buflen:self._buflen + length] = data
+        self._buf[self._buflen :] = data
         self._buflen += length
-        self.view = memoryview(self._buf[0:self._buflen])
+        self.view = memoryview(self._buf[0 : self._buflen])
 
     def reset(self) -> None:
         self._buflen = 0
@@ -387,11 +402,17 @@ class Buffer:
 
     def set(self, data: Union[bytes, bytearray, memoryview]) -> None:
         length = len(data)
-        if length > self._size:
-            raise BufferOverflow()
-        self._buf[0:length] = data
+        self._buf[0:] = data
         self._buflen = length
         self.view = memoryview(self._buf[0:length])
 
+    def get(self) -> bytearray:
+        val = self._buf[: self._buflen]
+        self.reset()
+        return val
+
     def __len__(self) -> int:
         return self._buflen
+
+    def __bytes__(self):
+        return bytes(self._buf[0 : self._buflen])
