@@ -508,11 +508,13 @@ class SevenZipFile(contextlib.AbstractContextManager):
         return_dict: bool = False,
         callback: Optional[ExtractCallback] = None,
     ) -> Optional[Dict[str, IO[Any]]]:
-        if callback is not None and not isinstance(callback, ExtractCallback):
-            raise ValueError("Callback specified is not a subclass of py7zr.callbacks.ExtractCallback class")
-        elif callback is not None:
+        if callback is None:
+            pass
+        elif isinstance(callback, ExtractCallback):
             self.reporterd = Thread(target=self.reporter, args=(callback,), daemon=True)
             self.reporterd.start()
+        else:
+            raise ValueError("Callback specified is not an instance of subclass of py7zr.callbacks.ExtractCallback class")
         target_files: List[Tuple[pathlib.Path, Dict[str, Any]]] = []
         target_dirs: List[pathlib.Path] = []
         if path is not None:
@@ -1013,11 +1015,6 @@ class SevenZipFile(contextlib.AbstractContextManager):
         elif isinstance(bio, io.TextIOBase):
             # First check whether is it Text?
             raise ValueError("Unsupported file object type: please open file with Binary mode.")
-        elif hasattr(bio, "read") and hasattr(bio, "__sizeof__"):
-            # CPython's io.BufferedIOBase or io.BufferedReader has __sizeof__, but
-            # pypy3 don't have. So first check __sizeof__ and then goes to alternative.
-            # Also allowing objet type which has read() and length methods for duck typing
-            size = bio.__sizeof__()
         elif isinstance(bio, io.BufferedIOBase):
             # come here when subtype of io.BufferedIOBase that don't have __sizeof__ (eg. pypy)
             # alternative for `size = bio.__sizeof__()`
